import matplotlib.pyplot as plt
import matplotlib.collections
import matplotlib.patches
import numpy as np
from matplotlib import rc


rc('text', usetex=True)
rc('text.latex', preamble="\n".join([
    '\\usepackage{amsmath}',
    '\\newcommand{\\D}{\mathrm{d}}'
]))


def errorband(p, ax=None, underflow=False, overflow=False):
    if ax is None:
        ax = plt.gca()

    p = p.copy()

    delta = p[2,0] - p[1,0]

    if p[0,0] == -np.inf:
        if underflow is False:
            p = p[1:]
        else:
            s = 2 if type(underflow) == bool else underflow
            p[0,0] = p[1,0] - (s+0.5) * delta
    if p[-1,0] == +np.inf:
        if overflow is False:
            p = p[:-1]
        else:
            s = 2 if type(overflow) == bool else overflow
            p[-1,0] = p[-2,0] + (s+0.5) * delta

    boxes = [
        matplotlib.patches.Rectangle(
            (x - delta, y - e/2), delta, e
        )
        for x, y, e in p
    ]

    artist = ax.step(p[:,0], p[:,1])
    col = artist[0].get_color()

    pc = matplotlib.collections.PatchCollection(
        boxes,
        facecolor=col, edgecolor=col
    )

    ax.add_collection(pc)
    return artist


def kplot(sigma, k, sigmar=[], kr=[],
          labelx='x_e',
          labelk="$K$", labelkr=None,
          labelsigma=None, labelsigmar=None):

    if type(sigma) == np.ndarray:
        sigma = [sigma]
    if type(sigmar) == np.ndarray:
        sigmar = [sigmar]
    if type(k) == np.ndarray:
        k = [k]
    if type(kr) == np.ndarray:
        kr = [kr]

    if labelsigma is None:
        labelsigma = r"$\D\sigma\,/\,\D " + labelx + "$"

    fig, axs = plt.subplots(2, sharex=True, sharey=True, gridspec_kw={'hspace': 0})
    axs[1].set_xlabel("$"+labelx+"$")

    axs[0].set_ylabel(labelsigma)
    for i in sigma:
        errorband(i, ax=axs[0])

    if len(sigmar) > 0:
        ax2 = axs[0].twinx()
        if labelsigmar is not None:
            ax2.set_ylabel(labelsigmar)
        for i in sigmar:
            errorband(i, ax=ax2)

    axs[1].set_ylabel(labelk)
    for i in k:
        errorband(i, ax=axs[1])

    if len(kr) > 0:
        ax3 = axs[1].twinx()
        if labelkr is not None:
            ax3.set_ylabel(labelkr)
        for i in kr:
            errorband(i, ax=ax3)

    return fig, axs
